/**
 * Command line interface. The CLI module 
 * provides a simple interface to parse arguments esaly. 
 * @type Module cli|Module cli
 */
var cli = require('cli');

/**
 * This Plugin is necessary to read a file, while an 
 * other slow process keeps still writting. 
 * Omnet++ writes its vector files chunk-whise. However, fs streamReader 
 * fires an end event per finished chunk, thus the pocess thinks we are at the
 * end of the file. This plugin fixed this issue.
 * @type Module growing-file|Module growing-file
 */
var GrowingFile = require('growing-file');
var fs = require('fs');

/**
 * The command line arguments are: 
 * - i The path to the existing vector file. 
 * - p The PID of ib_flit_sim. 
 * 
 * 
 * This script will kill ib_flit_sim. When the 
 * user presses a key, or a deadloack is detected.
 */
cli.parse({
    vector_file: ['i', 'Path to omnet++ vector file.' , 'string', 'results/One-0.vec'],
    pid: ['p', "PID of ib_flit_sim", 'string', 'unknown'],
    pathfilter: ['f', "Path to port filter file",'string','./portfilter.json']
});

/**
 * Evaluate data input stream until deadlock is found.
 * @type Compute 
 */
var comp = new Compute();

process.stdin.setRawMode(true);
process.stdin.resume();   
process.stdin.on('data', function(){
    comp.finish();
 // process.exit();
});

var max_time = 0;
var port_filter = null;
/**
 * Main body. 
 * - Bind all keyboard keys to close app, by securely releasing all resources.
 * - Read vector file line by line and parse the lines.
 * - Emit parsed object to computatio unit. 
 * - On finish cloase  file descriptor
 */
cli.main(function(args,options){
    // Stop when key prssed

    comp.ibfs_pid(options.pid);
 

    // Readline helps to transform the read buffer object into 
    // strings line by line. 
    var rl = require('readline').createInterface({
        terminal: false,
        input:  GrowingFile.open(options.vector_file,{
                timeout: 100000,
                interval: 500,
                startFromEnd: false
            })
    });
    
    if(fs.existsSync(options.pathfilter)){
        var contents = fs.readFileSync(options.pathfilter);
        // Define to JSON type
        port_filter = JSON.parse(contents);
    }   

    console.log("- Start dldec ... scanning!");
    console.log("-- ib_flit_sim PID: "+ options.pid);
    console.log("-- Omnet++ vector file: "+ options.vector_file);
    // Evaluate one vector line at a time
    rl.on('line', function (line) {
      
         // Stop reading when finished.
        if(comp.blockCall){
          rl.close();  
        };
        // parse string into an valid js data struct. 
        // on error, the object is false.
        var obj = parseVecLine( line );
        
        // Forward vaild item for compuation
        // `blockCall` interrupts futher compuation when app finished.
        if(obj && !comp.blockCall){
            comp.set(obj);
        }
       
    });
    
    rl.on('end',function(){
       console.log("end of readline"); 
    });

});



/**
 * Parse a line form the vector file. If the line is invalid, return false.
 * @param {type} line String repesentation of one line
 * @returns {parseHost.item|parseSwtich.item|parseVecLine.item}
 */
function parseVecLine( line ){
    var verbs = line.split(" ");
    //definition line	
    if(verbs[0].indexOf('vector') > -1){
// Example of a valid line
//verbs[0]    [ 'vector',
//verbs[1]    '82',
//verbs[2]    '',
//verbs[3]    'dlnet.SW_10.port[0].vlarb',
//verbs[4]     '',
//verbs[5]    '"free',
//verbs[6]    'credits',
//verbs[7]    'on',
//verbs[8]    'VL0"',
//verbs[9]    '',
//verbs[10]    'ETV' ]
            if(verbs[5].indexOf('"free') > -1 ){
                var is_switch = verbs[3].indexOf('SW_') > -1;
                if(is_switch){
                    return parseSwtich(verbs);
                }else{
                    return parseHost(verbs);
                }
            }
    }else{
            verbs = line.split("\t");
            if(verbs.length > 3){
                //verbs[0] [ '2', 
                //verbs[1] '219', 
                //verbs[2] '0.000000003', 
                //verbs[3] '0' ]
                 var item =  {
                    type : 1 , // data entry
                    id : parseInt(verbs[0]),
                    data: [ parseFloat(verbs[2]),  parseInt(verbs[3]) ]
                 };   
                 return item;
            }         
    }
    return false;
}

/**
 * Takes a verb array and models the swtich object item.
 * Each swtich event has an unique id. 
 * @param {type} verbs
 * @returns {parseSwtich.item}
 */
function parseSwtich(verbs){
    var domain = verbs[3].split('.');
    var item = {
        is_s: true,
        type:   0, 
        id :    parseInt(verbs[1]),
        name:   domain[1],
        port:   parseInt(domain[2].replace(/\D/g,'')),
        vl : verbs[8].substring(0, verbs[8].length - 1)
    };
    return item;
}

/**
 * Takes a verb array and models a host item. The host uses always port 0. 
 * @param {type} verbs
 * @returns {parseHost.item}
 */
function parseHost(verbs){
     var domain = verbs[3].split('.');
    var item = {
        type:   0,
        id :    parseInt(verbs[1]),
        name:   domain[1],
        port:   0, // has only one port
        vl: verbs[8].substring(0, verbs[8].length - 1)
    };
    return item;
} 


/**
 * Deadlock detection is done by via timeout. If the system doese not
 * fire any new events within two seconds, we consider it as a deadlock. 
 * @returns {Compute}
 */
function Compute(  ){
    this.ibfs_pid = function(pid){
        this.ib_flit_sim_PID = pid;
    };
    this.ib_flit_sim_PID = 'unknown';
    
    this.currentDate = 0;
    
    this.plot = new GNUPlot('plot/','ev_');
    
    this.idx = []; // stores all ids for fast filtering of data items
    this.end_of_definitions = true;
    
    this.credits = null;
    
    this.map ={};
    
    this.set = function( item ){
        // if switch or host
        if(item.type === 0 ){
            
            if(port_filter){
                if(port_filter[ item.name ] !== undefined 
                 && port_filter[item.name] ===  item.port 
                 && item.vl === 'VL0'
                 ){
                 
                    if(!this.credits){
                        this.credits = {};
                    }
                        
                    this.credits[ item.id ] = 255;
                    
                }else{
                    return 0;
                }
            }
            
           
            this.idx.push(item.id);
                  
            var title = item.name.replace("_", " ") + " P:"+item.port + ' ' + item.vl; 
            console.log(title);
            // create file streamwriter
            this.plot.newDataSet(item.id , title);
           
            this.map[ item.id ] = title;
           
          
            
        }else if(item.type === 1 && this.idx.indexOf(item.id) > -1 ){ 
            if(this.end_of_definitions){
                this.end_of_definitions = false;
                this.init();
            }
         
            var event_time = parseFloat(item.data[0]);
            if( event_time > max_time){
                max_time = event_time;
            }
         
           
            this.plot.emit( item.id, item.data );
          //  console.log( item.id + ' - ' + this.map[item.id] + " : " + item.data[1] );
            this.currentDate = (new Date()).getTime();
            
            if(this.credits ){
                this.credits[item.id] = item.data[1];
                var all_zero = true;
                for(var i in this.credits){
                    all_zero = all_zero & (this.credits[i] === 0);
                }
                if(all_zero){
                    if(!this.dltimer){
                        var me = this;
                        this.dltimer = setTimeout(function(){
                            clearInterval(this.timer);
                            me.finish("No Credits");
                        }, 10000);
                    }
                //   clearInterval(this.timer);
                //   this.finish("No Credits");
                }else{ 
                    if(this.dltimer){
                        clearTimeout(this.dltimer);
                        this.dltimer = null;
                    }
                }
            }
        }
    };
    this.dltimer = null;
    
    this.timer = null;
    this.init = function(){
        var me = this;
        this.currentDate = (new Date()).getTime();
        
        this.timer = setInterval(function(){
            if((new Date()).getTime() - me.currentDate > 20000){
                console.log("- Two seconsds elapsed without progress in network.");
                console.log("No response, after "+ max_time +"s of simulated real time.");
                me.finish("Timeout");
            }
        }, 2000);
    };
    
    
    this.blockCall =  false;
    this.finish = function( reason ){
       if(this.blockCall){return;}
       this.blockCall = true; // avoid closing streams two times in a row.
       clearInterval(this.timer);
        // close all stream witers
       console.log("- Closing streams (10s)");
       fs.writeFileSync('./deadlock.txt',"DL time("+reason+"): "+max_time);
       require('child_process').spawn('kill', [this.ib_flit_sim_PID]);

       this.plot.finish();
       this.plot.onEnd = function(){
            console.log("- All Streams are closed!");
            console.log("- Computation is finished.");
            process.exit();
       }
    };
      
}

/**
 * Generate a plot with GNU Plot. 
 * Frist, we need to create a new data set with `newDataSet`. It maps a `key`
 * and a `title` with the data set. A data set is a write stream which writes 
 * every new entry directly into the data file. A new item is stored with the 
 * `emit` method. The key parameter identifies where to store the item.
 * For finalization call `finish` to store all files. 
 * @param {string} folder The folder where to store the data files
 * @param {string} file_prefix The prefix name of the stored data files
 * @returns {GNUPlot}
 */
function GNUPlot( folder, file_prefix ){
    this.fs = require('fs');
    
    this.folder = folder;   
    this.file_prefix = file_prefix;
    
    this.titles = {}; // for each key store its title
    this.writers = {}; // store for each key a file stream writer 
    this.counter = 0;
    
    this.onEnd = function(){};
    
    this.emit_counter = 0;
    this.emit = function( key, data){
        var stream = this.writers[ key ];
        for(var d in data){
            stream.write( data[d] + ' ');
        }
        stream.write('\n'); 
        if(this.emit_counter++ === 100){
        //    stream.uncork();
          //  stream.cork();
            this.emit_counter = 0;
        }
            
    };
    
    this.newDataSet = function( key, title ){
        var me = this;
        this.counter++;
        this.writers[ key ] =  this.fs.createWriteStream(this.folder  + this.file_prefix + key +'.dat');
        this.writers[ key ].write('#'+ title +'\n');
        this.writers[ key ].write('#time data\n');
      //  this.writers[ key ].cork();
        this.writers[ key ].on('finish', function() {
            me.counter--;
            console.log('[Plot] Closing stream ' + me.counter );
            if(me.counter === 0){
                me.onEnd();
            }
        });
        
        this.titles[ key ] = title;  
    };
    
    this.finish = function(){
    
        // create gnuplot file 
        var content = "set terminal png size 1000,800 enhanced font \"Helvetica,9\"\n";
        content += 'set title "Free Credits on each Fabric Port"\n';
        content += 'set xlabel "Time [s]"\n';
        content += 'set ylabel "# Free Credits"\n';
        content += "set output 'output.png' \n";
        content += 'set termoption dash \n';
        content += "plot ";
      
        for(var key in this.titles){
            content += "\""+ this.folder + this.file_prefix + key +".dat\" using 1:2 title \""+this.titles[ key ]+"\" with lines,\\\n";
        }
        
        // remove last `,` (comma) and store the string to file.
        this.fs.writeFileSync("plot.plt", content.substring(0, content.length - 3)); 
        
        // close all stream witers
        for(var s in this.writers){
            this.writers[ s ].end('');
        }
    };
    
}