/**
 * Parent abstarct class for an application instance. 
 * Utilities are forked and thus called asynchroniously. 
 * An app has many events, std out, std err, and a close event. 
 * To call manay apps one after an other we need to call them in 
 * synchronious maner. To achive chain calls, we  a recursive 
 * call to itselfe by calling the next App in the chain when the close event 
 * is triggered. 
 * @param {App} app Next App in call chain, or null for no app.
 * @returns {nm$_app.App}
 */
function App( app ){
    /**
     * Next app in call chain.
     * @type App
     */
    this.app = app;
    
    /**
     * Start here the app. This is a abstract function for app initialization.
     * Within this function the app needs to call `callNext` in its close event
     * callback to run the next app in the call chain.
     * @param {object} options Pass vlaues to call the app correcly.
     * @returns {undefined}
     */
    this.start = function( options){
       return;   
    };
    
    /**
     * Call self. This function calls this app. Helpfull
     * on chain startup.
     * @param {object} options Pass vlaues to call the app correcly.
     * @returns {undefined}
     */
    this.callThis = function( options ){
        this.options = options;
        this.start( this.options );
    };
    
    /**
     * Call next App in the call chain. This functoin needs to be placed 
     * within the close callback of an app. It will invoke the `start` 
     * method of the next app.
     * @param {object} options Pass vlaues to call the app correcly.
     * @returns {undefined}
     */
    this.callNext = function( options ){
        if(this.app !== undefined && this.app !== null){
            this.app.start( options );
        }
    };
    
}

module.exports = App;