var App = require('./app');
var spawn = require('child_process').spawn;
var fs = require('fs');

function IBSwitches(app) {
    App.call(this, app);

    this.start = function (options) {
       
        console.log("--> Start ibswitches.");
        var me = this;
        
        var ibnetdiscover = spawn('ibswitches', 
                                    [], {env: process.env});

        var rl = require('readline').createInterface({
            terminal: false,
            input: ibnetdiscover.stdout
        });

        var lines = []
        rl.on('line', function (line) {
            lines.push(line);
        });
        
        rl.on('close', function () {
            if( (options.routing_name === 'updn' ||
            options.routing_name === 'dnup' ||
            options.routing_name === 'ftree' ||
            options.routing_name === 'torus-2QoS')){
                me.createRoutingConfigFile(options, lines);
            }
            me.callNext( options );
        });

        ibnetdiscover.stderr.on('data', function (data) {
            console.log('ibswitches stderr: ' + data);
        });

        ibnetdiscover.on('close', function (code) { /**/ });
    };

     // torus helper function
    this.get_device  = function (torus, pos){
        var res = torus;
        for(var i in pos){
            res = res[ pos[ i ] ];
        }
        return res;
    };

    this.createRoutingConfigFile = function (options, lines){
        var SwitchID_regExp = /\"SW\_([^\"]+)\"/;
        var SwitchGID_reqExp = /0x([^ ]+) /;
        var Lid_reqExp = /lid ([^ ]+) /;
        var SwitchByID = {};
        
        for(var i in lines){
            var line = lines[i];
            var id = SwitchID_regExp.exec(line)[1];
            SwitchByID[ id ] = {
                isHost: false,
                name: "SW_"+id,
                lid: parseInt(Lid_reqExp.exec(line)[1]),
                id: parseInt(id),
                gid: "0x"+SwitchGID_reqExp.exec(line)[1]
            };
            console.log(line);
        }
        if( options.routing_name === 'updn' ||
            options.routing_name === 'dnup' ||
            options.routing_name === 'ftree' ){
            for(var i in options.topology){
                if( !options.topology[ i ].is_host){// find first switch
                    var guid = SwitchByID[ options.topology[ i ].id ].gid;
                    fs.writeFileSync( './build/ibsimrootsw.txt', guid); 
                    return;
                }
            }
           
        }else if( options.routing_name === 'torus-2QoS'  ){
            if(options.torus && options.torusdim.length <= 3){
                var content = 'torus ';
                for(var i in options.torusdim){
                    content += options.torusdim[ i] + ' '
                }
                if( options.torusdim.length === 2){
                    content+= ' 1\n';
                }else{
                    content+= '\n';
                }
                 
                var dim = options.torusdim;
                dim.reverse();
                for( var n in options.topology){
                    if( ! options.topology[n].is_host ){
                        var sw = options.topology[n];
                        var base_gid = SwitchByID[ sw.id ].gid;
                        var pos = sw.pos;
                  
                        for(var i = 0; i < dim.length; i++ ){
                          
                            var up = pos.slice();
                            var down = pos.slice();
                            up[i] = (pos[ i ] + 1) % dim[i]; 
                            down[i] = (((pos[ i ] - 1) % dim[i]) + dim[i]) % dim[i];
                       
                            var up_gid = options.SwitchByID[ this.get_device( options.torus, up).id ].gid;
                            var down_gid = options.SwitchByID[ this.get_device( options.torus, down).id ].gid;
                            
                            if( i === 0){
                               content += 'xp_link ' + base_gid + ' ' + up_gid + '\n'; 
                               content += 'xm_link ' + base_gid + ' ' + down_gid + '\n'; 
                            }else if(i === 1){
                               content += 'yp_link ' + base_gid + ' ' + up_gid + '\n'; 
                               content += 'ym_link ' + base_gid + ' ' + down_gid + '\n'; 
                            }else if(i === 2){
                               content += 'zp_link ' + base_gid + ' ' + up_gid + '\n'; 
                               content += 'zm_link ' + base_gid + ' ' + down_gid + '\n';  
                            }
                            
                        }
                        break;
                    }
                }
                
                fs.writeFileSync( './build/torusconfig.txt', content);  
            }else{
                console.log( "-E- torus-2QoS needs a 2D/3D trous  topology."); 
                console.log( "-E- chnage routing to DFSSSP.");
                options.routing_name = 'dfsssp';
            }
        }      
    };
}

// JS OOP style to inherit methods from the parent class. 
// Set the "constructor" property to refer to this class.
IBSwitches.prototype = Object.create(App.prototype);
IBSwitches.prototype.constructor = IBSwitches;

module.exports = IBSwitches;