var App = require('./app');
var fs = require('fs');
var om_utils = require('../utils/omnetpp');

/**
 * Create omnet++ project folder. Project folder is the end result of 
 * dlgen. The directory contains the follwoing files. 
 *  - fdbs.txt          Routing tabels.
 *  - fdbs.ini          Map routing table to switch.
 *  - omnetpp.ini       The initial configuration for the simulator,
 *  - opensm.log        The log output of opensm when the network was configured.
 *  - run.sh            Run simulation in terminal with deadlock detection.
 *  - gui.sh            Run simulation in gui mode without deadlock detection.
 *  - modules.ini       Default configuration for switches and hosts.
 *  - pcie_8x_g2.ini    Default configuration for PCI sink by hosts.
 *  - dlnet.net         The main network file. 
 *  - host.dstSeq.ini   Map hosts to comunication pattern. 
 *  - host.dstSeq       Communication patterns for hosts.
 *  - host_sl2vl.txt    SL2VL tables of HCA in default configuration.
 *  - switch_sl2vl.txt  SL2VL tables of switches in default configuration.
 *  - host_sl2vl_df.txt SL2VL tables of HCA in DF configuration 
 *  - switch_sl2vl_df   SL2VL tables extracted by ibdiagnet. 
 * @param {App} app Next App to call. 
 * @returns {nm$_omnetpp.OMNeTpp}
 */
function OMNeTpp(app) {
    App.call(this, app);

    this.start = function (options) {
        var rootdir = options.config.omnet_workspace + options.project;
        console.log("--> Create omnet++ poject files.");

        console.log("--> Create Directory");
        fs.mkdirSync(rootdir, '0777');
        fs.chmodSync(rootdir , '777');
        console.log("--> Create Plot Directory");
        fs.mkdirSync(rootdir + '/plot', '0777');

        console.log("--> Write FDBS file");
        om_utils.create_FDBS_File(rootdir + "/fdbs.txt", options.routing);

        console.log("--> Write FDBS.ini file");
        om_utils.create_FDBS_ini_File(rootdir + "/fdbs.ini", options.routing);

        console.log("--> Query PathSL");
        var pathSL = om_utils.sl_routing(options);
        
        console.log("--> Write omnetpp.ini file");
        om_utils.create_omnet_File(rootdir + "/omnetpp.ini", "dlnet", pathSL); 

        console.log("--> Cpoy opensm log");
        om_utils.copy_File('./build/opensmtmp', rootdir + "/opensm.log");

        console.log("--> Cpoy run script");
        om_utils.copy_File('assets/run.sh', rootdir + "/run.sh", options);
        fs.chmodSync(rootdir + "/run.sh", '777');

        console.log("--> Cpoy default modules config");
        om_utils.copy_File('assets/modules.ini', rootdir + "/modules.ini");
        
        console.log("--> Cpoy default pcie config");
        om_utils.copy_File('assets/pcie_8x_g2.ini', rootdir + "/pcie_8x_g2.ini");
        
        console.log("--> Cpoy opensm config");
        om_utils.copy_File('opensm.conf', rootdir + "/opensm.conf");
        
        if(fs.existsSync('./build/portfilter.json')){
            console.log("--> Cpoy cycle path filter");
            om_utils.copy_File('./build/portfilter.json', rootdir + "/portfilter.json");
        }   

        console.log("--> Write Network file");
        om_utils.create_NED_File(rootdir + "/dlnet.ned", "dlnet", options.topology);

        console.log("--> Create desSeq ini file. ");
        om_utils.create_dst_sequence_ini_file(rootdir + "/host.dstSeq.ini", options.topology);

        console.log("--> Create desSeq  file. ");
        om_utils.create_dst_sequence_file(rootdir + "/host.dstSeq", options.dst_list);
        
        console.log("--> Create sl2vl files.");
        om_utils.create_sl2vl_file(rootdir , options);

        options.projectRootDir = rootdir;
        this.callNext( options );
    };

}

// JS OOP style to inherit methods from the parent class. 
// Set the "constructor" property to refer to this class.
OMNeTpp.prototype = Object.create(App.prototype);
OMNeTpp.prototype.constructor = OMNeTpp;

module.exports = OMNeTpp;
