var App = require('./app');
var fs = require('fs');
var spawn = require('child_process').spawn;
var GrowingFile = require('growing-file');

/**
 * Start opensm to generate routing and sl2vl tabels.
 * OpenSM is called with the following arguments:
 *  -F ./opensm.conf'               //Where to read config file. 
 *  -R  dfsssp                      //Name of the routing algorithm.
 *  -a ./build/ibsimrootsw.txt'     //Read in root LIDs. 
 *  -f ./build/opensmtmp'           //Wrtie opensm log file 
 *  -Q                              //Enable QoS
 *  -o                              //(optional)Close opensm on finished.
 * WARNING: the app is not terminating, because ibdiagnet needs to 
 * query for all path sl. Hence we continue with computation when the word 
 * "SUBNET UP"  apperas in the log.
 * @param {App} app Next app, which is called on success,
 * @returns {Communication}
 */
function OpenSM(app) {
    App.call(this, app);
    

    this.start = function (options) {
        console.log("--> Start opensm.");
        var me = this;
        
        fs.writeFileSync( './build/opensmtmp', ""); // empty log before starting.
        
        // To uses fat-tree or updn, one needs to specify 
        // one or more root nodes. 
        // The root node lid needs to be but in a spearate file.
        // Look up the opensmtmp file if errors occure
        // Enbale QoS with -Q: otherwise the sl2lv tabels ar not set. 
        var start = new Date();
        var opensm = spawn(options.config.opensm_path, 
        ['-F', './opensm.conf', 
         '-R', options.routing_name, 
         '--root_guid_file', './build/ibsimrootsw.txt',
         "--torus_config", './build/torusconfig.txt',
         '-f', './build/opensmtmp', 
         '-Q'
         //'-o'
        ], 
        {env: process.env});
        
        var rl  = require('readline').createInterface({
            terminal: false,
            input:  GrowingFile.open('./build/opensmtmp',{
                timeout: 3600000,
                interval: 500,
                startFromEnd: false
            })
        });
        
        var VL1_regExp = /needed:\s*([]+),*$/;
        var VL2_regExp = /VLs:\s([^)]+)\//;
        var vl;
        var logStream = fs.createWriteStream('log.txt', {'flags': 'a'});
        rl.on('line', function(line) {
            //console.log(line);
            // DFSSSP  'Virtual Lanes needed: 3'
            // LASH    'Lanes needed: 1,'
            // DF      'used VLs: 3/'
            
            if(line.indexOf("Lanes needed:") > 0){
                var end = new Date() - start;
                console.log(line);
                if(options.routing_name == 'dfsssp'){
                    vl = parseInt( /needed:\s*(\d+)$/.exec(line)[1]);
                    if(options.opensm == 1){
						logStream.write("\n"+options.routing_name +" "+options.topology_args+"\t\t"+ vl+"\t S:"+options.countS+"\t H:"+options.countH + "\t\t"+end+" ");
                    	logStream.end();
					}else if(options.opensm > 1){
						logStream.write(", "+end);
						logStream.end();
					}	

                }
                if(options.routing_name == 'lash'){
                    vl = parseInt( /needed:\s*(\d+),/.exec(line)[1]);
                    if(options.opensm == 1){
						logStream.write("\n"+options.routing_name +" "+options.topology_args+"\t\t"+ vl+"\t S:"+options.countS+"\t H:"+options.countH + "\t\t"+end+" ");
                    	logStream.end();
					}else if(options.opensm > 1){
						logStream.write(", "+end);
						logStream.end();
					}	
                }
                if(options.opensm){process.exit(0);}
            }else if(line.indexOf("used VLs:") > 0){
                var end = new Date() - start;
                console.log(line);
                
                vl = parseInt(VL2_regExp.exec(line)[1]);
                if(options.opensm == 1){
					logStream.write("\n"+options.routing_name +" "+options.topology_args+"\t\t"+ vl+"\t S:"+options.countS+"\t H:"+options.countH + "\t\t"+end+" ");
                	logStream.end();
				}else if(options.opensm > 1){
					logStream.write(", "+end);
					logStream.end();
				}	
                if(options.opensm){process.exit(0);}
            }else if(line.indexOf("ERR DF05") > 0){
                // no need to compute further
                console.log("-E- OpenSM cannot use DF routing.")
                if(options.opensm){process.exit(0);}
            }
            
            if(line.indexOf('SUBNET UP') > 0){
                rl.close();
            }
        });
        rl.on('close',function () {
            if(options.opensm == 0){
                me.callNext( options );
            }else{
                process.exit(0);
            }
        });
        opensm.stdout.on('data', function (data) {
           console.log(' ' + data);
        });
        
        opensm.stderr.on('data', function (data) {
            console.log('--> opensm stderr: ' + data);
        });
        opensm.on('close', function (code) {
            if (code === 0) {
                console.log('--> opensm is running.');
                //me.callNext( options );
            } else {
                console.log('--> opensm error code: ' + code);
            }
        });

    };
    
   
    
}

// JS OOP style to inherit methods from the parent class. 
// Set the "constructor" property to refer to this class.
OpenSM.prototype = Object.create(App.prototype);
OpenSM.prototype.constructor = OpenSM;

module.exports = OpenSM;
