#########################################################
#
#   Run omnetpp simulator. Type -h to see the help 
#   for more information.
#
#########################################################
usage="$(basename "$0") [-h] [-c n] [-t] -- program to autorun omnet++ in GUI or 
terminal mode (deadlock detection only in terminal mode). 

where:
    -h  show this help text
    -c  configuration (default: 42)
            - df deadlock free simulation
            - dl simulation without DF feature (default)
    -t  run in terminal with 
        dl dectection(default is GUI mode) 
"

TERMINAL_MODE="NO"
SIMCONFIG="dl"
while getopts ':hc:t' option; do
  case "$option" in
    h) echo "$usage"
       exit
       ;;
    c) SIMCONFIG=$OPTARG
       ;;
    :) printf "missing argument for -%c\n" "$OPTARG" >&2
       echo "$usage" >&2
       exit 1
       ;;
    t) TERMINAL_MODE="YES"
       ;;
   \?) printf "illegal option: -%s\n" "$OPTARG" >&2
       echo "$usage" >&2
       exit 1
       ;;
  esac
done
shift $((OPTIND - 1))


# Set binary
IBFLITSIM_ROOT=%%ibflitsim_root_dir%%
IBFLITSIM_BIN=%%ibflitsim_bin_path%%
# Include package paths
export NEDPATH=$IBFLITSIM_ROOT/src:./


# GUI MODE
if [ $TERMINAL_MODE = "NO" ]; then
    
    $IBFLITSIM_BIN -f omnetpp.ini -c $SIMCONFIG  

else
    #TERMINAL MODE
    SIMCONFIG=$SIMCONFIG"term"
   
    file=results/$SIMCONFIG-0.vec

    rm $file
    # Run simulation in command line mode
    $IBFLITSIM_BIN -f omnetpp.ini -c $SIMCONFIG -u Cmdenv &

    #Spin loop until vector file is created by simulator 
    while [ ! -f "$file" ]
    do
        inotifywait -qqt 2 -e create -e moved_to "$(dirname $file)" &> /dev/null
    done

    #Start deadlock detection 
    node %%dldec_path%% -i results/$SIMCONFIG-0.vec -p $(pidof ib_flit_sim)

    #end simulation when the deadlock detector has finished
    kill %1

    gnuplot plot.plt
fi