/**
 * Create a communication pattern where each host communicates 
 * with only one host, with: dlid = slid + hops mod maxID. 
 * The last host wraps back around to the first host in the topology list. 
 * Suitable for ring topology
 * 
 * @example node dlgen -c jumps -g 3 
 * Each host cummincates with an host, which is 
 * has dlid = sid + 3. This means three hops in the 
 * ring topology.
 * @param {object} topology The topology array. 
 * @param {int} hops  Destination LID offest.
 * @returns {nm$_jumps.Jumps}
 */
function Jumps( options , hops ){
    var check = /\d+/;
    var OK = check.exec(hops);  
    if( !OK ){
        console.log('-E- Jumps pattern has invalid argument: '+hops);
        console.log('-E- Use the following options: -g <number>');
        console.log('-E- Abort.');
        process.exit( 0 );
    }
    
    this.topology = options.topology;
    this.hops = parseInt(hops);
    this.max_host = 0;
    
    //Check input
    
    
    // store the communication pattern of each host
    this.list = {}; 
    
    this.get_max_host_id = function ( ){
        var max = 0;
        for(var i in this.topology){
            var id =  this.topology[i].id;
            if( id > max &&  this.topology[i].is_host ){
                max = id;
            }
        }
        return max;
    };
    
    this.compute = function(){
        this.max_host = this.get_max_host_id();
        for(var i in this.topology){
            if(  this.topology[i].is_host ){
                var host = this.topology[i];
                
                if( this.list[ host.id ] === undefined ){
                    this.list[ host.id ] = [];
                }
                
                // ibsim dose not support to have hosts with lid = 0. Thus host
                // indexing beginns with 1.
                this.list[ host.id ].push( ((host.id - 1 + this.hops) % (this.max_host )) + 1 );
            }
        } 
    };
    
    this.compute();
};


module.exports = Jumps;
