/**
 * Create a random comunication pattern. One-to-One communication 
 * between random host.
 * 
 * @example node dlgen -c random
 * 
 * @param {object} topology The network topology
 * @param {void} nothing This argument will be ignored
 * @returns {nm$_random.Random}
 */
function Random( options , nothing ){
    this.topology = options.topology;
 
    this.max_host = 0;
    this.list = {}; // store the communication pattern of each host
    
    this.get_max_host_id = function ( ){
        var max = 0;
        for(var i in this.topology){
            var id =  this.topology[i].id;
            if( id > max &&  this.topology[i].is_host ){
                max = id;
            }
        }
        return max;
    };
    
  
    
    this.compute = function(){
        this.max_host = this.get_max_host_id();
        for(var i in this.topology){
            if(  this.topology[i].is_host ){
                var host = this.topology[i];
                
                var hid = Math.floor((Math.random() * this.max_host ) + 1); 
                while(hid === host.id){
                    // avoid self sending.
                    hid = Math.floor((Math.random() * this.max_host ) + 1); 
                }
                
                if( this.list[ host.id ] === undefined ){
                    this.list[ host.id ] = [];
                }
                
                this.list[ host.id ].push( hid );
            }
        } 
    };
    
    this.compute();
};


module.exports = Random;
