/**
 * Representation of a IB Device. 
 * Host and Switches inherit from Device.
 * @param {type} id
 * @returns {nm$_device.Device}
 */
function Device( id ){
    
    // the device id
    this.id = id;
    
    // the device name
    this.name =  "D_" + id;
    
    // total number of ports
    this.max_ports = 0;  

    // array of conntected devices
    // the index indicates the port number
    this.port = [];
    
    // if device is a channel adapter (host)
    this.is_host  = false;
    
    // add new device, and automatically increase 
    // the port array. 
    this.add = function( device , port){
        //dont self conntect
        if(device === this ||  device.name === this.name ){return;}
        
        // dont connect if already connected
        if(this.is_connected(device)){return;}
        if(port !== undefined){
            this.port[ port ] = device;
            this.max_ports++;
            return;
        }
        
        // connect devices
        this.port[ this.max_ports++ ] = device;
        device.port[ device.max_ports++ ] = this;
    };
    
    this.setName = function(name){
        this.name = name;
    };
    
    // return device name
    this.toString = function(){
        return this.name;
    };
    
    //check if a device is already conntected
    this.is_connected= function(device){
        return this.port.indexOf( device ) > -1;
    }
}

module.exports = Device;