// In order to use ibsim, we need to preload the 
// libumad simulation library for every ibutility and opesm. 
// Example: sudo LD_PRELOAD=/usr/lib/umad2sim/libumad2sim.so opensm

process.env.PATH =   '/home/ottob/2015-ib-deadlocks/opensm-3.3.19/bin/sbin:/home/ottob/2015-ib-deadlocks/opensm-3.3.19/bin/lib:' + process.env.PATH ;
process.env.OSM_TMP_DIR="./tmp";
process.env.OSM_CACHE_DIR="./tmp";
process.env.SIM_HOST="H_1";

var cli = require('cli');
var fs = require('fs');

var Topology = require('./app/topology');
var Communication = require('./app/communication');
var IBSim  = require('./app/ibsim');
var ibswitches = require('./app/ibswitches');
var OpenSM = require('./app/opensm');
var dump_lfts = require('./app/dump_lfts');
var ibnetdiscover = require('./app/ibnetdiscover');
var ibdiagnet = require('./app/ibdiagnet');
var omnetpp = require('./app/omnetpp');
var info = require('./app/info');

// White lists to santizie user input.
var allowed_topologies = [
    'file', 
    'torus', 
    'ring', 
    'kgraph', 
    'fat-tree', 
    'custom'
];

var allowed_routing_engines = [
    'minhop',
    'updn',
    'dnup',
    'ftree',
    'lash',
    'dor',
    'dfsssp',
    'sssp'
];

var allowed_pattern = [
    'deadlock',
    'jumps',
    'random',
    'stencil'
];


cli.parse({
    project:     ['p', 'Name of the omnet++ project folder' , 'string', 'exp_dragon_fly_sssp'],
    
    topology_name:['t', 'Name of the topology generator, ' +
                        'e.g file, torus, ring, kgraph, fat-tree, custom', 'string','file'],
            
    topology_args:['a', 'Topology arguments','string', '2;DF.2.edges.txt'],
    
    communication_pattern : ['c','Communication pattern of the hosts, ' + 
                'e.g. deadlock, jumps, random, stencil', 'string', 'deadlock'],
    
    communication_args : ['g' , 'Pattern arguments', 'string', '8'],
    
    routing_name:     ['r', 'Routing engine from OpenSM','string' ,'sssp'],
    
    exit : ['x','Exit dlgen after project has been created ? ', 'bool', true],
    opensm : ['o','Exit dlgen after Opensm has completed.', 'number', 0],
});

/*
 *  ibnetdiscover
 * cat /tmp/ibdiagnet.slvl 
 */
/* How to get the sl2vl mapping table  
 * 
 * // not pretty
 * sudo LD_PRELOAD=/usr/lib/umad2sim/libumad2sim.so ibdiagnet -r -vlr
 * cat /tmp/ibdiagnet.fdbs
 * cat /tmp/ibdiagnet.slvl 
 * // better
 * 
 * sudo LD_PRELOAD=/usr/lib/umad2sim/libumad2sim.so smpquery sl2vl 1
 * 
 * 
 * View vlarb table 
 * sudo LD_PRELOAD=/usr/lib/umad2sim/libumad2sim.so smpquery vlarb 5 3
 *                                                               lid port
 */


cli.main(function(args,options){
    
    // create build directory if not found
    if(!fs.existsSync( './build' )){
        fs.mkdirSync( './build', '0777' );
    }
    
    // Test if config file exists
    if(!fs.existsSync('./config.json')){
        console.log('-E- No config file found.');
        console.log('-E- Abort.');
        process.exit( 0 );
    }
    
    options.config =  require('./config.json');
  
    process.env.LD_PRELOAD = options.config.libumad2sim_path;
  
    // Try to generate workspace folder
    if( !fs.existsSync(options.config.omnet_workspace) ){
        fs.mkdirSync( options.config.omnet_workspace, '0777' );
    }
  
    // Test if project name exists
    if(fs.existsSync( options.config.omnet_workspace + options.project )){
        console.log("-E- Project folder '"+ options.config.omnet_workspace +
                    options.project +"' exists already.");
        console.log('-E- Abort.');
        process.exit( 0 );
    }
    
    //White list filter on topology option
    if( allowed_topologies.indexOf(options.topology_name) < 0 ){
        console.log('-E- Topology with name '+options.topology_name+' not found.');
        console.log('-E- Please try the following names:'+allowed_topologies);
        console.log('-E- Abort.');
        process.exit( 0 );
    }
    
    //White list filter on routing option
    if( allowed_routing_engines.indexOf(options.routing_name) < 0 ){
        console.log('-E- Routing engine with name '+options.routing_name+' not found.');
        console.log('-E- Please try the following names: '+allowed_routing_engines);
        console.log('-E- Abort.');
        process.exit( 0 );
    }
    
    //White list filter on pattern option
    if( allowed_pattern.indexOf(options.communication_pattern) < 0 ){
        console.log('-E- Pattern with name '+options.communication_pattern+' not found.');
        console.log('-E- Please try the following names: '+allowed_pattern);
        console.log('-E- Abort.');
        process.exit( 0 );
    }
  
    new Topology(               // 1) Create Topology
    new IBSim(                  // 2) Start ibsim and load network.
    new ibswitches(             // 3) Get switch GID
    new OpenSM(                 // 4) Start opensm
    new ibnetdiscover(          // 5) Discover network to get the real lid
    new dump_lfts(              // 6) Get routing tabels
    new Communication(          // 7) Create pattern
    new ibdiagnet(              // 8) Extract SL2VL tables; 
    new omnetpp(                // 9) Create Omnet++ project directory.
    new info(null)))))))))      
    ,options);
    


});




