var Switch = require('../components/switch');
var Host = require('../components/host');

/**
 * Collection of costum topologies. The simple graph format is an array 
 * containing of litral object, which are nodes. 
 * A host has the form {host: true , edges[1,2]}, where 'host: true' indicates 
 * that the node is a HCA. The 'edges; array containes indicies. The indecies are
 * the graph array indecies of other nodes. A switch has only the eges array
 * { edges[1,2]}. A supplimentary name field can be added for readablility,
 * but has no effect. Swtiches will be always named with a SW_ prefix and 
 * hosts with the H_ prefix.
 * 
 * At the end do not forget to add the graph to the 'list' dictionary.
 * 
 * An example call looks like this:
 * @example node dlgen -t custom -a ssspexample
 * We load the custom graph called 'ssspexample' which is part of the list 
 * and has a reference to the graph with the variable name 'paperSSSP'
 */

var custom_topology = [
    {// 0 
 
        edges: [1, 2, 3, 6, 9, 21, 33],
        name: "SW_0" 
    },
    {// 1 
        host: true,
        edges: [0],
        name: "H_1" 
    },
    {// 2 
        host: true,
        edges: [0],
        name: "H_2" 
    },
    {// 3 
 
        edges: [0, 4, 5, 6, 9, 42, 54],
        name: "SW_0" 
    },
    {// 4 
        host: true,
        edges: [3],
        name: "H_3" 
    },
    {// 5 
        host: true,
        edges: [3],
        name: "H_4" 
    },
    {// 6 
 
        edges: [0, 3, 7, 8, 9, 63, 75],
        name: "SW_0" 
    },
    {// 7 
        host: true,
        edges: [6],
        name: "H_5" 
    },
    {// 8 
        host: true,
        edges: [6],
        name: "H_6" 
    },
    {// 9 
 
        edges: [0, 3, 6, 10, 11, 84, 96],
        name: "SW_0" 
    },
    {// 10 
        host: true,
        edges: [9],
        name: "H_7" 
    },
    {// 11 
        host: true,
        edges: [9],
        name: "H_8" 
    },
    {// 12 
 
        edges: [13, 14, 15, 18, 21, 33, 45],
        name: "SW_0" 
    },
    {// 13 
        host: true,
        edges: [12],
        name: "H_9" 
    },
    {// 14 
        host: true,
        edges: [12],
        name: "H_10" 
    },
    {// 15 
 
        edges: [12, 16, 17, 18, 21, 54, 66],
        name: "SW_0" 
    },
    {// 16 
        host: true,
        edges: [15],
        name: "H_11" 
    },
    {// 17 
        host: true,
        edges: [15],
        name: "H_12" 
    },
    {// 18 
 
        edges: [12, 15, 19, 20, 21, 75, 87],
        name: "SW_0" 
    },
    {// 19 
        host: true,
        edges: [18],
        name: "H_13" 
    },
    {// 20 
        host: true,
        edges: [18],
        name: "H_14" 
    },
    {// 21 
 
        edges: [0, 12, 15, 18, 22, 23, 96],
        name: "SW_0" 
    },
    {// 22 
        host: true,
        edges: [21],
        name: "H_15" 
    },
    {// 23 
        host: true,
        edges: [21],
        name: "H_16" 
    },
    {// 24 
 
        edges: [25, 26, 27, 30, 33, 45, 57],
        name: "SW_0" 
    },
    {// 25 
        host: true,
        edges: [24],
        name: "H_17" 
    },
    {// 26 
        host: true,
        edges: [24],
        name: "H_18" 
    },
    {// 27 
 
        edges: [24, 28, 29, 30, 33, 66, 78],
        name: "SW_0" 
    },
    {// 28 
        host: true,
        edges: [27],
        name: "H_19" 
    },
    {// 29 
        host: true,
        edges: [27],
        name: "H_20" 
    },
    {// 30 
 
        edges: [24, 27, 31, 32, 33, 87, 99],
        name: "SW_0" 
    },
    {// 31 
        host: true,
        edges: [30],
        name: "H_21" 
    },
    {// 32 
        host: true,
        edges: [30],
        name: "H_22" 
    },
    {// 33 
 
        edges: [0, 12, 24, 27, 30, 34, 35],
        name: "SW_0" 
    },
    {// 34 
        host: true,
        edges: [33],
        name: "H_23" 
    },
    {// 35 
        host: true,
        edges: [33],
        name: "H_24" 
    },
    {// 36 
 
        edges: [37, 38, 39, 42, 45, 57, 69],
        name: "SW_0" 
    },
    {// 37 
        host: true,
        edges: [36],
        name: "H_25" 
    },
    {// 38 
        host: true,
        edges: [36],
        name: "H_26" 
    },
    {// 39 
 
        edges: [36, 40, 41, 42, 45, 78, 90],
        name: "SW_0" 
    },
    {// 40 
        host: true,
        edges: [39],
        name: "H_27" 
    },
    {// 41 
        host: true,
        edges: [39],
        name: "H_28" 
    },
    {// 42 
 
        edges: [3, 36, 39, 43, 44, 45, 99],
        name: "SW_0" 
    },
    {// 43 
        host: true,
        edges: [42],
        name: "H_29" 
    },
    {// 44 
        host: true,
        edges: [42],
        name: "H_30" 
    },
    {// 45 
 
        edges: [12, 24, 36, 39, 42, 46, 47],
        name: "SW_0" 
    },
    {// 46 
        host: true,
        edges: [45],
        name: "H_31" 
    },
    {// 47 
        host: true,
        edges: [45],
        name: "H_32" 
    },
    {// 48 
 
        edges: [49, 50, 51, 54, 57, 69, 81],
        name: "SW_0" 
    },
    {// 49 
        host: true,
        edges: [48],
        name: "H_33" 
    },
    {// 50 
        host: true,
        edges: [48],
        name: "H_34" 
    },
    {// 51 
 
        edges: [48, 52, 53, 54, 57, 90, 102],
        name: "SW_0" 
    },
    {// 52 
        host: true,
        edges: [51],
        name: "H_35" 
    },
    {// 53 
        host: true,
        edges: [51],
        name: "H_36" 
    },
    {// 54 
 
        edges: [3, 15, 48, 51, 55, 56, 57],
        name: "SW_0" 
    },
    {// 55 
        host: true,
        edges: [54],
        name: "H_37" 
    },
    {// 56 
        host: true,
        edges: [54],
        name: "H_38" 
    },
    {// 57 
 
        edges: [24, 36, 48, 51, 54, 58, 59],
        name: "SW_0" 
    },
    {// 58 
        host: true,
        edges: [57],
        name: "H_39" 
    },
    {// 59 
        host: true,
        edges: [57],
        name: "H_40" 
    },
    {// 60 
 
        edges: [61, 62, 63, 66, 69, 81, 93],
        name: "SW_0" 
    },
    {// 61 
        host: true,
        edges: [60],
        name: "H_41" 
    },
    {// 62 
        host: true,
        edges: [60],
        name: "H_42" 
    },
    {// 63 
 
        edges: [6, 60, 64, 65, 66, 69, 102],
        name: "SW_0" 
    },
    {// 64 
        host: true,
        edges: [63],
        name: "H_43" 
    },
    {// 65 
        host: true,
        edges: [63],
        name: "H_44" 
    },
    {// 66 
 
        edges: [15, 27, 60, 63, 67, 68, 69],
        name: "SW_0" 
    },
    {// 67 
        host: true,
        edges: [66],
        name: "H_45" 
    },
    {// 68 
        host: true,
        edges: [66],
        name: "H_46" 
    },
    {// 69 
 
        edges: [36, 48, 60, 63, 66, 70, 71],
        name: "SW_0" 
    },
    {// 70 
        host: true,
        edges: [69],
        name: "H_47" 
    },
    {// 71 
        host: true,
        edges: [69],
        name: "H_48" 
    },
    {// 72 
 
        edges: [73, 74, 75, 78, 81, 93, 105],
        name: "SW_0" 
    },
    {// 73 
        host: true,
        edges: [72],
        name: "H_49" 
    },
    {// 74 
        host: true,
        edges: [72],
        name: "H_50" 
    },
    {// 75 
 
        edges: [6, 18, 72, 76, 77, 78, 81],
        name: "SW_0" 
    },
    {// 76 
        host: true,
        edges: [75],
        name: "H_51" 
    },
    {// 77 
        host: true,
        edges: [75],
        name: "H_52" 
    },
    {// 78 
 
        edges: [27, 39, 72, 75, 79, 80, 81],
        name: "SW_0" 
    },
    {// 79 
        host: true,
        edges: [78],
        name: "H_53" 
    },
    {// 80 
        host: true,
        edges: [78],
        name: "H_54" 
    },
    {// 81 
 
        edges: [48, 60, 72, 75, 78, 82, 83],
        name: "SW_0" 
    },
    {// 82 
        host: true,
        edges: [81],
        name: "H_55" 
    },
    {// 83 
        host: true,
        edges: [81],
        name: "H_56" 
    },
    {// 84 
 
        edges: [9, 85, 86, 87, 90, 93, 105],
        name: "SW_0" 
    },
    {// 85 
        host: true,
        edges: [84],
        name: "H_57" 
    },
    {// 86 
        host: true,
        edges: [84],
        name: "H_58" 
    },
    {// 87 
 
        edges: [18, 30, 84, 88, 89, 90, 93],
        name: "SW_0" 
    },
    {// 88 
        host: true,
        edges: [87],
        name: "H_59" 
    },
    {// 89 
        host: true,
        edges: [87],
        name: "H_60" 
    },
    {// 90 
 
        edges: [39, 51, 84, 87, 91, 92, 93],
        name: "SW_0" 
    },
    {// 91 
        host: true,
        edges: [90],
        name: "H_61" 
    },
    {// 92 
        host: true,
        edges: [90],
        name: "H_62" 
    },
    {// 93 
 
        edges: [60, 72, 84, 87, 90, 94, 95],
        name: "SW_0" 
    },
    {// 94 
        host: true,
        edges: [93],
        name: "H_63" 
    },
    {// 95 
        host: true,
        edges: [93],
        name: "H_64" 
    },
    {// 96 
 
        edges: [9, 21, 97, 98, 99, 102, 105],
        name: "SW_0" 
    },
    {// 97 
        host: true,
        edges: [96],
        name: "H_65" 
    },
    {// 98 
        host: true,
        edges: [96],
        name: "H_66" 
    },
    {// 99 
 
        edges: [30, 42, 96, 100, 101, 102, 105],
        name: "SW_0" 
    },
    {// 100 
        host: true,
        edges: [99],
        name: "H_67" 
    },
    {// 101 
        host: true,
        edges: [99],
        name: "H_68" 
    },
    {// 102 
 
        edges: [51, 63, 96, 99, 103, 104, 105],
        name: "SW_0" 
    },
    {// 103 
        host: true,
        edges: [102],
        name: "H_69" 
    },
    {// 104 
        host: true,
        edges: [102],
        name: "H_70" 
    },
    {// 105 
 
        edges: [72, 84, 96, 99, 102, 106, 107],
        name: "SW_0" 
    },
    {// 106 
        host: true,
        edges: [105],
        name: "H_71" 
    },
    {// 107 
        host: true,
        edges: [105],
        name: "H_72" 
    },
];


var randomg = [
    {// 0 
        host: true,
        edges: [6], // 0       
        name: "H0"
    },
    {// 1 
        host: true,
        edges: [7], // 0   
        name: "H1"
    },
    {// 2
        host: true,
        edges: [8], // 0   
        name: "H2"
    },
    {// 3 
        host: true,
        edges: [9], // 0   
        name: "H3"
    },
    {// 4
        host: true,
        edges: [10], // 0   
        name: "H4"
    },
    {// 5 
        host: true,
        edges: [11], // 0   
        name: "H5"
    },
    { // 6

        edges: [0, 7, 11], // 2 
        name: "A"
    },
    { // 7

        edges: [1, 10, 8, 6], // 2 
        name: "B"
    },
    { // 8

        edges: [2, 7, 9], // 2 
        name: "C"
    },
    { // 9

        edges: [3, 10, 8], // 2 
        name: "D"
    },
    { // 10

        edges: [4, 7, 11, 9], // 2 
        name: "E"
    },
    { // 11

        edges: [5, 10, 6], // 2 
        name: "F"
    },
];


var s2h2 = [
    {// 0 
        host: true,
        edges: [1], // 0   
        name: "H0"
    }, { // 1

        edges: [0, 2], // 1
        name: "A"
    }, { // 2

        edges: [1, 3], // 2 
        name: "B"
    }, {// 3 
        host: true,
        edges: [2], // 0   
        name: "H1"
    }];

var timooriginal = [
    {
        host: true,
        edges: [6],
        name: "H0" // 0 
    }, {
        host: true,
        edges: [],
        name: "H1" // 1
    }, {
        host: true,
        edges: [13],
        name: "H2" // 2
    }, {
        host: true,
        edges: [],
        name: "H3" // 3
    }, {
        host: true,
        edges: [],
        name: "H4" // 4
    }, {
        host: true,
        edges: [10],
        name: "H5" // 5
    }, {
        edges: [1, 7],
        name: "S0" // 6
    }, {
        edges: [6, 11],
        name: "S1" // 7
    }, {
        edges: [7],
        name: "S2" // 8
    }, {
        edges: [8, 10],
        name: "S3" // 9
    }, {
        edges: [4, 9],
        name: "S4" // 10
    }, {
        edges: [12],
        name: "S5" // 11
    }, {
        edges: [14, 13],
        name: "S6" // 12
    }, {
        edges: [3, 12],
        name: "S7" // 13
    }, {
        edges: [9],
        name: "S8" // 14
    }];

var timofullconnected = [
    {
        host: true,
        edges: [6],
        name: "H0" // 0 
    }, {
        host: true,
        edges: [6],
        name: "H1" // 1
    }, {
        host: true,
        edges: [13],
        name: "H2" // 2
    }, {
        host: true,
        edges: [13],
        name: "H3" // 3
    }, {
        host: true,
        edges: [10],
        name: "H4" // 4
    }, {
        host: true,
        edges: [10],
        name: "H5" // 5
    }, {
        edges: [0, 1, 7],
        name: "S0" // 6
    }, {
        edges: [6, 11, 8],
        name: "S1" // 7
    }, {
        edges: [7, 9],
        name: "S2" // 8
    }, {
        edges: [8, 10, 14],
        name: "S3" // 9
    }, {
        edges: [4, 5, 9],
        name: "S4" // 10
    }, {
        edges: [12, 7],
        name: "S5" // 11
    }, {
        edges: [14, 13, 11],
        name: "S6" // 12
    }, {
        edges: [2, 3, 12],
        name: "S7" // 13
    }, {
        edges: [9, 12],
        name: "S8" // 14
    }
]

var paperSSSP = [
    {// 0 
        host: true,
        edges: [3], // 0   
        name: "H0"
    }, {//1
        host: true,
        edges: [7], // 1
        name: "H1"
    }, {//2
        host: true,
        edges: [5], // 2 
        name: "H2"
    }, {//3
        edges: [0, 4, 6], // A 
        name: "A"
    }, {//4
        edges: [3, 5, 7], // B
        name: "B"
    }, {//5
        edges: [2, 6, 4], // C
        name: "C"
    }, {//6
        edges: [3, 5, 7], // D
        name: "D"
    }, {//7
        edges: [1, 4, 6], // E 
        name: "E"
    }];

var custom_topology2 = [
    {// 0 
 
        edges: [1, 2, 3, 6, 9, 21, 33],
        name: "SW_0" 
    },
    {// 1 
        host: true,
        edges: [0],
        name: "H_1" 
    },
    {// 2 
        host: true,
        edges: [0],
        name: "H_2" 
    },
    {// 3 
 
        edges: [4, 5, 0, 6, 9, 42, 54],
        name: "SW_1" 
    },
    {// 4 
        host: true,
        edges: [3],
        name: "H_3" 
    },
    {// 5 
        host: true,
        edges: [3],
        name: "H_4" 
    },
    {// 6 
 
        edges: [7, 8, 0, 3, 9, 63, 75],
        name: "SW_2" 
    },
    {// 7 
        host: true,
        edges: [6],
        name: "H_5" 
    },
    {// 8 
        host: true,
        edges: [6],
        name: "H_6" 
    },
    {// 9 
 
        edges: [10, 11, 0, 3, 6, 84, 96],
        name: "SW_3" 
    },
    {// 10 
        host: true,
        edges: [9],
        name: "H_7" 
    },
    {// 11 
        host: true,
        edges: [9],
        name: "H_8" 
    },
    {// 12 
 
        edges: [13, 14, 15, 18, 21, 33, 45],
        name: "SW_4" 
    },
    {// 13 
        host: true,
        edges: [12],
        name: "H_9" 
    },
    {// 14 
        host: true,
        edges: [12],
        name: "H_10" 
    },
    {// 15 
 
        edges: [16, 17, 12, 18, 21, 54, 66],
        name: "SW_5" 
    },
    {// 16 
        host: true,
        edges: [15],
        name: "H_11" 
    },
    {// 17 
        host: true,
        edges: [15],
        name: "H_12" 
    },
    {// 18 
 
        edges: [19, 20, 12, 15, 21, 75, 87],
        name: "SW_6" 
    },
    {// 19 
        host: true,
        edges: [18],
        name: "H_13" 
    },
    {// 20 
        host: true,
        edges: [18],
        name: "H_14" 
    },
    {// 21 
 
        edges: [22, 23, 0, 12, 15, 18, 96],
        name: "SW_7" 
    },
    {// 22 
        host: true,
        edges: [21],
        name: "H_15" 
    },
    {// 23 
        host: true,
        edges: [21],
        name: "H_16" 
    },
    {// 24 
 
        edges: [25, 26, 27, 30, 33, 45, 57],
        name: "SW_8" 
    },
    {// 25 
        host: true,
        edges: [24],
        name: "H_17" 
    },
    {// 26 
        host: true,
        edges: [24],
        name: "H_18" 
    },
    {// 27 
 
        edges: [28, 29, 24, 30, 33, 66, 78],
        name: "SW_9" 
    },
    {// 28 
        host: true,
        edges: [27],
        name: "H_19" 
    },
    {// 29 
        host: true,
        edges: [27],
        name: "H_20" 
    },
    {// 30 
 
        edges: [31, 32, 24, 27, 33, 87, 99],
        name: "SW_10" 
    },
    {// 31 
        host: true,
        edges: [30],
        name: "H_21" 
    },
    {// 32 
        host: true,
        edges: [30],
        name: "H_22" 
    },
    {// 33 
 
        edges: [34, 35, 0, 12, 24, 27, 30],
        name: "SW_11" 
    },
    {// 34 
        host: true,
        edges: [33],
        name: "H_23" 
    },
    {// 35 
        host: true,
        edges: [33],
        name: "H_24" 
    },
    {// 36 
 
        edges: [37, 38, 39, 42, 45, 57, 69],
        name: "SW_12" 
    },
    {// 37 
        host: true,
        edges: [36],
        name: "H_25" 
    },
    {// 38 
        host: true,
        edges: [36],
        name: "H_26" 
    },
    {// 39 
 
        edges: [40, 41, 36, 42, 45, 78, 90],
        name: "SW_13" 
    },
    {// 40 
        host: true,
        edges: [39],
        name: "H_27" 
    },
    {// 41 
        host: true,
        edges: [39],
        name: "H_28" 
    },
    {// 42 
 
        edges: [43, 44, 3, 36, 39, 45, 99],
        name: "SW_14" 
    },
    {// 43 
        host: true,
        edges: [42],
        name: "H_29" 
    },
    {// 44 
        host: true,
        edges: [42],
        name: "H_30" 
    },
    {// 45 
 
        edges: [46, 47, 12, 24, 36, 39, 42],
        name: "SW_15" 
    },
    {// 46 
        host: true,
        edges: [45],
        name: "H_31" 
    },
    {// 47 
        host: true,
        edges: [45],
        name: "H_32" 
    },
    {// 48 
 
        edges: [49, 50, 51, 54, 57, 69, 81],
        name: "SW_16" 
    },
    {// 49 
        host: true,
        edges: [48],
        name: "H_33" 
    },
    {// 50 
        host: true,
        edges: [48],
        name: "H_34" 
    },
    {// 51 
 
        edges: [52, 53, 48, 54, 57, 90, 102],
        name: "SW_17" 
    },
    {// 52 
        host: true,
        edges: [51],
        name: "H_35" 
    },
    {// 53 
        host: true,
        edges: [51],
        name: "H_36" 
    },
    {// 54 
 
        edges: [55, 56, 3, 15, 48, 51, 57],
        name: "SW_18" 
    },
    {// 55 
        host: true,
        edges: [54],
        name: "H_37" 
    },
    {// 56 
        host: true,
        edges: [54],
        name: "H_38" 
    },
    {// 57 
 
        edges: [58, 59, 24, 36, 48, 51, 54],
        name: "SW_19" 
    },
    {// 58 
        host: true,
        edges: [57],
        name: "H_39" 
    },
    {// 59 
        host: true,
        edges: [57],
        name: "H_40" 
    },
    {// 60 
 
        edges: [61, 62, 63, 66, 69, 81, 93],
        name: "SW_20" 
    },
    {// 61 
        host: true,
        edges: [60],
        name: "H_41" 
    },
    {// 62 
        host: true,
        edges: [60],
        name: "H_42" 
    },
    {// 63 
 
        edges: [64, 65, 6, 60, 66, 69, 102],
        name: "SW_21" 
    },
    {// 64 
        host: true,
        edges: [63],
        name: "H_43" 
    },
    {// 65 
        host: true,
        edges: [63],
        name: "H_44" 
    },
    {// 66 
 
        edges: [67, 68, 15, 27, 60, 63, 69],
        name: "SW_22" 
    },
    {// 67 
        host: true,
        edges: [66],
        name: "H_45" 
    },
    {// 68 
        host: true,
        edges: [66],
        name: "H_46" 
    },
    {// 69 
 
        edges: [70, 71, 36, 48, 60, 63, 66],
        name: "SW_23" 
    },
    {// 70 
        host: true,
        edges: [69],
        name: "H_47" 
    },
    {// 71 
        host: true,
        edges: [69],
        name: "H_48" 
    },
    {// 72 
 
        edges: [73, 74, 75, 78, 81, 93, 105],
        name: "SW_24" 
    },
    {// 73 
        host: true,
        edges: [72],
        name: "H_49" 
    },
    {// 74 
        host: true,
        edges: [72],
        name: "H_50" 
    },
    {// 75 
 
        edges: [76, 77, 6, 18, 72, 78, 81],
        name: "SW_25" 
    },
    {// 76 
        host: true,
        edges: [75],
        name: "H_51" 
    },
    {// 77 
        host: true,
        edges: [75],
        name: "H_52" 
    },
    {// 78 
 
        edges: [79, 80, 27, 39, 72, 75, 81],
        name: "SW_26" 
    },
    {// 79 
        host: true,
        edges: [78],
        name: "H_53" 
    },
    {// 80 
        host: true,
        edges: [78],
        name: "H_54" 
    },
    {// 81 
 
        edges: [82, 83, 48, 60, 72, 75, 78],
        name: "SW_27" 
    },
    {// 82 
        host: true,
        edges: [81],
        name: "H_55" 
    },
    {// 83 
        host: true,
        edges: [81],
        name: "H_56" 
    },
    {// 84 
 
        edges: [85, 86, 9, 87, 90, 93, 105],
        name: "SW_28" 
    },
    {// 85 
        host: true,
        edges: [84],
        name: "H_57" 
    },
    {// 86 
        host: true,
        edges: [84],
        name: "H_58" 
    },
    {// 87 
 
        edges: [88, 89, 18, 30, 84, 90, 93],
        name: "SW_29" 
    },
    {// 88 
        host: true,
        edges: [87],
        name: "H_59" 
    },
    {// 89 
        host: true,
        edges: [87],
        name: "H_60" 
    },
    {// 90 
 
        edges: [91, 92, 39, 51, 84, 87, 93],
        name: "SW_30" 
    },
    {// 91 
        host: true,
        edges: [90],
        name: "H_61" 
    },
    {// 92 
        host: true,
        edges: [90],
        name: "H_62" 
    },
    {// 93 
 
        edges: [94, 95, 60, 72, 84, 87, 90],
        name: "SW_31" 
    },
    {// 94 
        host: true,
        edges: [93],
        name: "H_63" 
    },
    {// 95 
        host: true,
        edges: [93],
        name: "H_64" 
    },
    {// 96 
 
        edges: [97, 98, 9, 21, 99, 102, 105],
        name: "SW_32" 
    },
    {// 97 
        host: true,
        edges: [96],
        name: "H_65" 
    },
    {// 98 
        host: true,
        edges: [96],
        name: "H_66" 
    },
    {// 99 
 
        edges: [100, 101, 30, 42, 96, 102, 105],
        name: "SW_33" 
    },
    {// 100 
        host: true,
        edges: [99],
        name: "H_67" 
    },
    {// 101 
        host: true,
        edges: [99],
        name: "H_68" 
    },
    {// 102 
 
        edges: [103, 104, 51, 63, 96, 99, 105],
        name: "SW_34" 
    },
    {// 103 
        host: true,
        edges: [102],
        name: "H_69" 
    },
    {// 104 
        host: true,
        edges: [102],
        name: "H_70" 
    },
    {// 105 
 
        edges: [106, 107, 72, 84, 96, 99, 102],
        name: "SW_35" 
    },
    {// 106 
        host: true,
        edges: [105],
        name: "H_71" 
    },
    {// 107 
        host: true,
        edges: [105],
        name: "H_72" 
    },
];



var list = {
    "randomg" : randomg,
    "timooriginal" : timooriginal,
    "timofull": timofullconnected,
    "ssspexample" : paperSSSP,
    "custom_topology" : custom_topology,
    "custom_topology2" : custom_topology2,
};

/*
function Custom(x, options) {

    var G = list[x];
    if(G === undefined){
        console.log("-E- Custom graph not found.");
        console.log("-E- Try: "+Object.keys(list));
        process.exit(0);
    }

    var topo = [];
    var hi = 1;
    var si = 0;

    for (var i in G) {
        if (G[i].host === undefined) {
            topo.push(new Switch(si++));
        } else {
            topo.push(new Host(hi++));
        }
    }

    for (var i in G) {
        for (var j in G[i].edges) {
            topo[i].add(topo[ G[ i ].edges[ j ] ]);
        }
    }

    return topo;
}


module.exports = Custom;
*/

function Custom(x, options) {

    var G = list[x];
    if(G === undefined){
        console.log("-E- Custom graph not found.");
        console.log("-E- Try: "+Object.keys(list));
        process.exit(0);
    }

    var topo = [];
    var hi = 1;
    var si = 0;

    for (var i in G) {
        if (G[i].host === undefined) {
            topo.push(new Switch(si++));
        } else {
            topo.push(new Host(hi++));
        }
    }

    for (var i in G) {
        for (var j in G[i].edges) {
            topo[i].add(topo[ G[ i ].edges[ j ] ],j);
            topo[i].setName(G[ i ].name);
        }
    }

    return topo;
}

module.exports = Custom;

