var Switch= require('../components/switch');
var Host  = require('../components/host');

var sid = 0;
var hid = 1;
/**
 * Generate a fat tree topology. The simulator does not suppurt 
 * multiple links to one device, hence we  get a binary tree 
 * topology.
 * 
 * @example node dlgen -t fat-tree -a 5,3
 * 
 * Create a three level fat-tree, and add 5 host to each leaf switch.
 * 
 * @param {type} args Host per switch and Level ( h,l )
 * @returns {undefined}
 */
function Fat_tree(args , options){
    
    var check = /\d+,\d+/;
    var OK = check.exec(args);  
    if( !OK ){
        console.log('-E- Fat-Tree has invalid argument: '+args);
        console.log('-E- Use the following options: -a <number>,<number>');
        console.log('-E- Abort.');
        process.exit( 0 );
    }
    
    var parts = args.split(",");
    var level = parts[1];
    var hosts_per_swtich = parts[0];
    var topology = [];
    var root_swtich = new  Switch(sid++);
    add_devices(topology, root_swtich, level, hosts_per_swtich);
    return topology;
}

function add_devices( topology, root_switch , level , hosts_per_swtich){
    topology.push( root_switch );
    
    if( level === 0){
        // add hosts
        for(var i = 0; i < hosts_per_swtich; i++){
            var host = new  Host(hid++);
            root_switch.add( host );
            topology.push( host );
        }
        return;
    }
    
    //create switch 1 
    var swtich1 = new  Switch(sid++);
    //create switch 2
    var swtich2 = new  Switch(sid++);
    
    root_switch.add(swtich1);
    root_switch.add(swtich2);
    
    add_devices(topology, swtich1 ,level - 1 , hosts_per_swtich);
    add_devices(topology, swtich2 ,level - 1 , hosts_per_swtich);
}


module.exports = Fat_tree;