var Switch = require('../components/switch');
var Host   = require('../components/host');

/**
 * Generate a fully connected graph. Each host is connected to exactly
 * one switch. The switches are fully connected between each other.
 * 
 * @example node dlgen -t kgraph -a 10
 * 
 * A fully connected graph with 10 hosts and switches. 
 * 
 * @param {integer} args number of Host
 * @returns {Array|K.topology|nm$_kgraph.K.topology}
 */
function K( args , options ){
    var check = /\d+/;
    var OK = check.exec(args);  
    if( !OK ){
        console.log('-E- K graph has invalid argument: '+args);
        console.log('-E- Use the following options: -a <number>');
        console.log('-E- Abort.');
        process.exit( 0 );
    }
    var n_switches = parseInt(args);
    var n_hosts = parseInt(args);
    var hNodes = [];
    var sNodes = [];
    for(var i = 0; i < n_hosts; i++){
        hNodes.push( new  Host( i + 1 ) );
    }
    // we want unique ids for all emelents
    for(var i = 0; i < n_switches; i++){
        sNodes.push( new  Switch(i));
    }
    
    // connect host in round robin 
    // and set comunication partner
    for(var i = 0; i < n_hosts; i++){
        sNodes[ i % n_switches].add( hNodes[i] );
    }
    // connect all-to-all switches 
    for(var i = 0; i < n_switches; i++){      
        var currSwitch = sNodes[i];
        for(var j = 0; j < n_switches; j++){
        
            var nextSwitch = sNodes[j];
            currSwitch.add( nextSwitch );
        } 
    }
    
    var topology = [];
    
    for(var h in hNodes){
        topology.push( hNodes[ h ] );
    }
    
    for(var s in sNodes){
        topology.push( sNodes[ s ] );
    }
    
    return topology;
}

module.exports = K;