var Switch = require('../components/switch');
var Host   = require('../components/host');

/**
 * Generate a Ring topology. 
 * 
 * @example node dlgen -t ring -a 4x5
 * 
 * Ring topology with 5 switches and 4 host per switch.
 * 
 * @param {integer} args  Number of host per switch and number of switch. ( {host}x{switch} )
 * @returns {undefined}
 */
function Ring( args , options){
    
    var check = /\d+x\d+/;
    var OK = check.exec(args);  
    if( !OK ){
        console.log('-E- Ring topology has invalid argument: '+args);
        console.log('-E- Use the following options: -a <number>x<number>');
        console.log('-E- Abort.');
        process.exit( 0 );
    }
    
    var parts = args.split("x");
    var n_switches = parts[1];
    var n_hosts = parts[0] * n_switches;
    var hNodes = [];
    var sNodes = [];
    for(var i = 0; i < n_hosts; i++){
        hNodes.push( new  Host( i + 1 ) );
    }
    // we want unique ids for all emelents
    for(var i = 0; i < n_switches; i++){
        sNodes.push( new  Switch(i));
    }
    
    // connect host in round robin 
    // and set comunication partner
    for(var i = 0; i < n_hosts; i++){
        sNodes[ i % n_switches].add( hNodes[i] );
    }
    // connect switches in cycle
    for(var i = 0; i < n_switches; i++){        
        var currSwitch = sNodes[i];
        var nextSwitch = sNodes[(i+1) % n_switches];
        currSwitch.add( nextSwitch );
    }
    
    var topology = [];
    
    for(var h in hNodes){
        topology.push( hNodes[ h ] );
    }
    
    for(var s in sNodes){
        topology.push( sNodes[ s ] );
    }
    
    return topology;
}

module.exports = Ring;