var Switch = require('../components/switch');
var Host  = require('../components/host');

var sid = 0;
var hid = 1;

/**
 * Tours topology. Each switch has one host.
 * 
 * @example node dlgen -t torus -a 3x4x5
 * 
 * Create a 3D torus with 60 hosts. 
 *    
 * @param {type} args
 * @returns {Torus.topology|Array}
 */
function Torus( args ,options ){
    
    var check = /\d+x\d+(?:x\d+)*/;
    var OK = check.exec(args);  
    if( !OK ){
        console.log('-E- Torus has invalid argument: '+args);
        console.log('-E- Use the following options, e.g. 4x4x5 (no space between \'x\'): -a <number>x<number>x(...)x<number>');
        console.log('-E- Abort.');
        process.exit( 0 );
    }
    
    // parse input
    var part = args.split('x');
    var dim = [];
    for(var i in part){
        dim.push( parseInt( part[i] ) );
    }
    // create help structure
    var topology = [];
    var torus = make_grid(topology, dim.reverse() );
    
    options.torus = torus;
    options.torusdim = dim;
    
    // conntect links between switches
    dim.reverse();
    for( var n in topology){
        if( ! topology[n].is_host ){
            var sw = topology[n];
           
            var pos = sw.pos;
           // console.log( "=> "+sw.name+ " : "+ pos);
            for(var i = 0; i < dim.length; i++ ){
                // the links are bidriectional, 
                // the wrap round is the same for all sites
                var up = pos.slice();
                up[i] = (pos[ i ] + 1) % dim[i]; 
             //   console.log(up);
                sw.add( get_device( torus, up));
 
            }
        }
    }
    return topology;
}


function make_grid(topology, dim, lvl, arr , pos) {
  if (lvl === 0) return add_devices(topology, dim , pos);
  if (!lvl) lvl = dim.length;
  if (!arr) arr = [];
  if (!pos) pos = new Array( dim.length );
  
  for (var i = 0, l = dim[lvl - 1]; i < l; i += 1) {
    pos[ dim.length - lvl  ] = i;  
    arr[i] = make_grid(topology, dim, lvl - 1, arr[i], pos);
  }
  
  return arr;
}

function add_devices(topology,dim, pos){
  
    var host = new  Host( hid++ );
    var host1 = new  Host( hid++ );
    var sw   = new Switch( sid++ );
    host.pos = pos.slice();
    host1.pos = pos.slice();
    sw.pos = pos.slice();
    sw.add(host);
    sw.add(host1);
    
    topology.push(host);
    topology.push(host1);
    topology.push(sw);
    
    return sw;
}

function get_device(torus, pos){
    var res = torus;
    for(var i in pos){
        res = res[ pos[ i ] ];
    }
    return res;
}

module.exports = Torus;