Generate OMNeT++ InfiniBand Simulation Project
========================================================

Mellanox Inc. offers an InfiniBand (IB) flit simulator extension 
for OMNeT++. The setup of a working simulation is very tedious. 
One has to define by hand the topology and routing tables, which is 
, however, not feasible for large topologies. 

This script, which is called 'dlgen' (Deadlock generator), helps 
to build OMNeT++ IB projects automatically. It generates all necessary files 
and configurations to run the OMNeT++ IB simulation out of the box. dlgen has a library 
for different topologies and communication patterns, which helps the user to build fast large networks. This saves time to create new experiments.   

The main scope of dlgen is to support the research of credit induced deadlocks in IB. 
It is hard to identify potential deadlock scenarios. dlgen helps to setup any arbitrary topology and finds automatically a potential deadlock. When a deadlock is found, dlgen creates the necessary communication patterns to exploit the deadlock. The results are stored as OMNetT++ files. In the end, we use OMNeT++ to performer the actual simulation and analyze the results. 
 

### Requirements 

The following software modules need to be installed in order to create a correct 
OMNeT++ project folder.

- OMNeT++
- ibsim
- opensm
- ib-flit-sim (from this repository, because it supports SL2VL mapping) 
- dump_lfts 
- ibnetdiscover
- ibdiagnet
- nodejs

### Installation

Standard installation of OMNeT++. If you get problems, please consult the official installation guide: https://omnetpp.org/doc/omnetpp/InstallGuide.pdf

	$ cd ~/
	$ # download the the source code of OMNeT++
	$ tar xvfz omnetpp-4.6-src.tgz
	$ cd omnetpp-5.0b3
    $ . setenv
	$ ./configure
	$ make 

Installation of `ibsim`: 

	$ apt-get install ibsim-utils

Installation of `OpenSM`:

	$ apt-get install opensm

To use the deadlock free network extension, you need to install the OpenSM version in the repository by hand.
In order to avoid  conflicts with an already installed OpenSM,  
we force our OpenSM to save its binaries in its installation folder.

	$ ./configure --prefix=./bin --exec-prefix=./bin --enable-debug 
	$ make -f Makefile install


Installation of `ib-flit-sim`: (suppose you are reading this README in the dlgen folder) 

	$ cd ../ib_flit_sim.28_07_13
	$ export PATH=$PATH:$HOME/omnetpp-4.6/bin #store in ~/.bashrc
	$ opp_makemake -f --deep -O out -X utils
	$ make makefiles 
	$ make 
	$ make MODE=release

Installation of `dump_lfts` and `ibnetdiscover`:

	$ apt-get install infiniband-diags

Installation of `ibdiagnet`:

	$ apt-get install ibutils

Installation of `nodejs`:

    $ apt-get install nodejs  

####Installation of `dlgen`:

	$ cd ../dlgen
	$ npm install --save

Open `config.json` and set all paths to the application we installed before.
A possible configuration can look like this:

```javascript
{
    "libumad2sim_path"  : "/usr/lib/umad2sim/libumad2sim.so",
    "ibsim_path"        : "/usr/sbin/ibsim",
    "opensm_path"       : "../opensm-3.3.19/bin/sbin/opensm" , 
    "dump_lfts_path"    : "/usr/sbin/dump_lfts",
    "ibnetdiscover_path": "/usr/sbin/ibnetdiscover",
    "omnet_workspace"   : "../mysimulations/",
    "ibflitsim_root_dir": "../../ib_flit_sim.28_07_13",
    "ibflitsim_bin_path": "$IBFLITSIM_ROOT/out/gcc-release/src/ib_flit_sim",
    "dldec_path"        : "../../dldec/dldec",
    "ibdiagnet_path"    : "/usr/sbin/ibdiagnet"
}   

```

The attributes have the following meaning:

- `libumad2sim_path`: Path to the libumad of ibsim.
- `ibsim_path`: Path to ibsim itself.
- `opensm_path`: Path to OpenSM. For the propose of the thesis, use the opensm proved in the repository. It has the extended deadlock free module.
- `dump_lfts_path`: Path to dump_lfts. It prints out all routing tables.
- `ibnetdiscover_path`: Path to ibnetdiscover.
- `omnet_workspace`: Define a folder where dlgen stores the generated OMNeT++ project folders. 
- `ibflitsim_root_dir`: The path of ib-flit-sim root directory relative to the OMNeT++ project folder. If in doubt, you can use an absolute path. 
- `ibflitsim_bin_path`: The exact location of the ib-flit-sim binary.
- `dldec_path`: Path to the deadlock detector.
- `ibdiagnet_path`:Path to ibdiagnet.

## Running dlgen

You can read the usage by typing: 

	$ nodejs index -h

	Usage:
  		index [OPTIONS] [ARGS]

	Options: 
  		-p, --project [STRING] Name of the omnet++ project folder 
  		-t, --topology_name [STRING]Name of the topology generator, e.g slimfly, 
                              torus, ring, kgraph, fat-tree, custom 
  		-a, --topology_args [STRING]Topology arguments 
  		-c, --communication_pattern [STRING]Communication pattern of the hosts, 
                                      e.g. deadlock, jumps, random, 
                                      stencil 
  		-g, --communication_args [STRING]Pattern arguments
  		-r, --routing_name [STRING]Routing engine from OpenSM 
  		-x, --exit [BOOL]      Exit dlgen after project has been created ?
  		-h, --help             Display help and usage details

### Hands-on Example

Suppose we use the `config.json` above. 
The follow call will generate a ring topology with 5 switches connected in ring and 2 hosts connected per switch. 
Furthermore it uses the 'deadlock' communication pattern to automatically find a channel dependency cycle and force the hosts 
to exploit the possible deadlock situation. Last, we select the 'dfsssp' routing engine to setup the routing tables of the switches.  

	$ cd <to dlgen folder>
	$ nodejs index -p MyTestProject -t ring -a 2x5 -c deadlock -r dfsssp

dlgen has generated the OMNeT++ project. You can find it in ../mysimulations/MyTestProject.
Next, we run the simulation. 

	$ cd ../mysimulations/MyTestProject
	$ ./run.sh

The simulator starts up and you are free to test the network until it deadlocks. 

### Documentation

#### Topologies

- `-t fat-tree` Build a fat-tree, but it is more like a binary tree. Multiple uplinks are not supported, because of ibsim. The fat-tree algorthims needs also the argument `-a <number A>,<number B>`: where number A is number of hosts per leaf switch, and number B is the tree depth, e.g. `-a 5,3` is a binary tree with 5 hosts pers leafs and a tree depth of three, hence a total of 2^3x5 = 40 hosts.

- `-t file` It Loads an adjacent list file form the `./topologies/file` folder. We need the `-a <number>;<filename>` argument to indicate the number of hosts and name of the file. Each node of the adjacent list is parsed as a switch. For example `-t file -a 4;DF.2.edges.txt` will parse the file `./topologies/file/DF.2.edges.txt`, and add to each switch 4 hosts. 

- `-t kgraph` This module creates a K graph topology of switches. Each switch has one host attached to it. Use the arguments `-a <number>` to indicate the number of switches in the graph.

- `-t ring` Create a ring topology. The argument `-a <number>x<number>` is necessary to determine the number of switches and the number of hosts per switch.  For example, `-t ring -a 3x5` will create a ring of 5 switches and 3 hosts per switch. 

- `- t torus` Create a n-Dimensional torus of switches. Use `-a <number>x<number>x...x<number>` to define the dimension of the torus. We attach 2 hosts per switch. For example, `-t torus -a 3x4x5` creates 3D torus with the depth 3, 4, and 5. We get 60 switches with an total of 120 hosts. 

### Communication pattern

- `-c deadlock` Search for a deadlock possibility in any arbitrary topology and force hosts to communicate along the cycle. No argument for `-g` is needed.

- `-c jumps` This communication pattern is best suited for the ring topology. `-g n` tells a source host to communicate with an host n hops away (only for ring topology). 

- `-c random` Random communication pattern, no `-g` argument necessary.

- `-c stencil`  Communication pattern of a basic stencil in a multidimensional torus. 
 	This are the special n-point stencil. The communication pattern is independent 
 	of the real topology. All host are arranged in a fictive torus. If the topology is a torus, then 
 	the both tori overlap perfectly. 
 	The size of the host tours must have enough place to fit  all hosts. For example, `-c stencil -g 2,3x5` forces the hosts to communicate like they were in an 3x5 2D torus, with a 7 point stencil pattern. 
  
               *
               *
           * * 0 * *
               *
               *


### Routing

We specifiy the routing engine with argument `-r <name>`, where 'name' indicates the routing engine used by OpenSM.
You can use the following routing engines: 'minhop', 'updn', 'dnup', 'ftree', 'lash', 'dor', 'dfsssp', and 'sssp'.

### Running Simulations

When we run simulation, we can chose from a set arguments: 

```
run.sh [-h] [-c n] [-t] -- program to autorun omnet++ in GUI or 
terminal mode (deadlock detection only in terminal mode). 

where:
    -h  show this help text
    -c  configuration (default: 42)
            - df deadlock free simulation
            - dl simulation without DF feature (default)
    -t  run in terminal with 
        dl detection(default is GUI mode) 

```

We can chose from two different configurations:

#### Deadlock Configuration: 

The simulation is run without the mechanisms of deadlock-freedom. 
Run the simulation in GUI mode and without event logging: 

	$ ./run.sh

This is the same as:

	$ ./run.sh -c dl
	 
Run the simulation in Terminal mode with event logging and automatic deadlock detection activated (uses automatically dldec):

	$ ./run.sh -t 

This is the same as:

	$ ./run.sh -c dl -t

#### Deadlock-free Configuration: 

The simulation is run with deadlock-free routing. 
Run the simulation in GUI mode and without event logging: 

	$ ./run.sh -c df
	 
Run the simulation in Terminal mode with event logging and automatic deadlock detection activated (uses automatically dldec):

	$ ./run.sh -c df -t 

Notice that this simulation will not deadlock: If you want to stop the simulation in terminal mode, press any key.




